package com.luojigou.xiaodou.ble

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.luojigou.xiaodou.R
import com.luojigou.xiaodou.XDScanBLEService
import com.luojigou.xiaodou.XDScanBLEService.Companion.ACTION_SCAN_RESULT
import com.luojigou.xiaodou.XDScanBLEService.Companion.ACTION_SCAN_RESULT_DATA
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.MainScope

@SuppressLint("MissingPermission")
class XDScanBLENormalFragment(private val host: XDScanBLEStatus.Host) : Fragment(),
    CoroutineScope by MainScope() {
    private val adapter = XDScanBLEAdapter {
        //        launch {
        //            host.getManager().connectWifi(it.device, "zdzh", "zdzh00000")
        //        }
    }

    private lateinit var selectWifi: ActivityResultLauncher<Intent>

    private lateinit var bluetoothReceiver: BroadcastReceiver
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_scanble_normal, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        selectWifi = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {

        }

        val button = view.findViewById<Button>(R.id.scan_again)
        button.setOnClickListener { restartScan() }

        val recyclerView = view.findViewById<RecyclerView>(R.id.scan_list)
        val layoutManager = GridLayoutManager(requireContext(), 2, RecyclerView.VERTICAL, false)

        recyclerView.layoutManager = layoutManager
        recyclerView.adapter = adapter
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        startService()
    }

    private fun startService() {
        bluetoothReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent) {
                when (intent.action) {
                    ACTION_SCAN_RESULT -> {
                        val list = intent.getParcelableArrayListExtra(
                            ACTION_SCAN_RESULT_DATA, XDScanBLEDevice::class.java
                        )

                        if (list != null) {
                            setAdapterValue(list)
                        }
                    }
                }
            }
        }

        val intentFilter = IntentFilter()
        intentFilter.addAction(ACTION_SCAN_RESULT)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            requireContext().registerReceiver(
                bluetoothReceiver,
                intentFilter,
                Context.RECEIVER_NOT_EXPORTED
            )
        } else {
            requireContext().registerReceiver(
                bluetoothReceiver,
                intentFilter,
            )
        }


        requireContext().startForegroundService(
            Intent(
                requireContext(), XDScanBLEService::class.java
            )
        )
    }

    private fun stopService() {
        requireContext().stopService(
            Intent(
                requireContext(), XDScanBLEService::class.java
            )
        )
    }

    private fun restartScan() {
        setAdapterValue(listOf())
        requireActivity().sendBroadcast(
            Intent(
                XDScanBLEService.ACTION_START_SCAN
            )
        )
    }

    private fun setAdapterValue(list: List<XDScanBLEDevice>) {
        adapter.setValues(list)
    }

    override fun onDetach() {
        stopService()
        super.onDetach()
    }
}